<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 *
 * FILOBLU - LOGIC CHANGED
 * @author - JORDY DE ROSA - jordy.derosa@filoblu.com
 */


namespace Amasty\Coupons\Observer;

use Magento\Framework\Event\ObserverInterface;
use Amasty\Coupons\Model\DiscountCollector;
use Magento\Framework\Event\Observer;
use Magento\SalesRule\Model\Rule;
use Amasty\Coupons\Helper\Data;


/**
 * Class CouponObserver
 */
class CouponObserver implements ObserverInterface
{
    /**
     * @var DiscountCollector
     */
    protected $discountCollector;
    protected $saleRule;
    protected $helperData;


    /**
     * CouponObserver constructor.
     * @param DiscountCollector $discountCollector
     * @param Rule $saleRule
     * @param Data $data
     */
    public function __construct(
        DiscountCollector $discountCollector,
        Rule $saleRule,
        Data $data
    )
    {
        $this->discountCollector = $discountCollector;
        $this->saleRule = $saleRule;
        $this->helperData = $data;
    }


    /**
     * @param Observer $observer
     */
    public function execute(Observer $observer)
    {
        if($observer->getRule()->getId())
        {
            //reset amount to override discount
            $amount = 0;
            $observer->getResult()->setAmount($amount);

            //get current rule data
            $ruleId = $observer->getRule()->getId();
            $rule = $this->saleRule->load($ruleId);
            $ruleSimpleAction= $rule->getSimpleAction() ?? '';
            $ruleDiscount = $rule->getDiscountAmount() ?? 0;

            //count how many same rule coupons are applied to the quote beacuse same rule coupons doesn't trigger observer
            $appliedCodes = explode(",",$observer->getData('quote')->getCouponCode());

            $ruleDescription = $this->helperData->getRuleLabelById($ruleId);
            if(!is_null($observer->getRule()->getCode()))
            {
                $sameRuleCounter = $this->helperData->countSameRuleCoupons($appliedCodes,$ruleId)['count'];
                $ruleUsedCodes = $this->helperData->countSameRuleCoupons($appliedCodes,$ruleId)['coupons'];
            }
            else
            {
                $sameRuleCounter =1;
                $ruleUsedCodes = "";
            }

            if($ruleDescription == "")
            {
                $ruleDescription = $ruleUsedCodes;
            }

            $item = $observer->getItem();
            $discount = $this->helperData->calculateDiscount($ruleSimpleAction,$ruleDiscount*$sameRuleCounter,$item,$observer);

            //register discount description
            $this->discountCollector->applyRuleAmount(str_replace(","," ",$ruleDescription),0);
            $observer->getResult()->setAmount($discount);
        }
    }

}
